import aiohttp
import pytest
from aioresponses.core import aioresponses

from lunatone_rest_api_client import Auth, Device, Devices
from lunatone_rest_api_client.models import (
    DeviceData,
    DevicesData,
    DeviceUpdateData,
)

from .common import DEVICES_DATA_RAW


@pytest.fixture
def devices_data() -> DevicesData:
    return DevicesData.model_validate(DEVICES_DATA_RAW)


@pytest.fixture
def device_data(devices_data: DevicesData) -> DeviceData:
    for d in devices_data.devices:
        return d


@pytest.mark.asyncio
async def test_devices_update(
    aioresponses: aioresponses, base_url: str, devices_data: DevicesData
) -> None:
    json_data = devices_data.model_dump(by_alias=True)
    aioresponses.get(f"{base_url}/{Devices.path}", payload=json_data)

    async with aiohttp.ClientSession() as session:
        devices = Devices(Auth(session, base_url))
        await devices.async_update()
        assert devices.data == devices_data


@pytest.mark.asyncio
async def test_device_update(
    aioresponses: aioresponses, base_url: str, device_data: DevicesData
) -> None:
    json_data = device_data.model_dump(by_alias=True)
    aioresponses.get(
        f"{base_url}/{Device.base_path}/{device_data.id}", payload=json_data
    )

    async with aiohttp.ClientSession() as session:
        device = Device(
            Auth(session, base_url),
            DeviceData(
                id=device_data.id, address=device_data.address, line=device_data.line
            ),
        )
        await device.async_update()
        assert device.data == device_data


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "name, groups",
    [
        ("Test Device 1 (Updated)", [0, 3, 6, 8]),
        ("Test Device 2 (Updated)", [2, 5, 9, 15]),
    ],
)
async def test_device_update_without_changes(
    aioresponses: aioresponses,
    base_url: str,
    device_data: DeviceData,
    name: str,
    groups: list[int],
) -> None:
    old_name = device_data.name
    old_groups = device_data.groups
    device_data.name = name
    device_data.groups = groups
    json_data = device_data.model_dump(by_alias=True)
    aioresponses.put(
        f"{base_url}/{Device.base_path}/{device_data.id}", payload=json_data
    )

    async with aiohttp.ClientSession() as session:
        device_data.name = old_name
        device_data.groups = old_groups
        device = Device(Auth(session, base_url), device_data)
        await device.async_update(DeviceUpdateData(name=name, groups=groups))
        assert device.data.name == name
        assert device.data.groups == groups



from typing import Any

from aiohttp import ClientSession, ClientResponse


class Auth:
    """Class to make authenticated requests."""

    def __init__(
        self, session: ClientSession, base_url: str, access_token: str | None = None
    ) -> None:
        """Initialize the auth."""
        self._session = session
        self._base_url = base_url
        self._access_token = access_token

    async def request(
        self, method: str, path: str, **kwargs: dict[str, Any]
    ) -> ClientResponse:
        """Make a request."""
        headers = kwargs.get("headers")

        if headers is None:
            headers = {}
        else:
            headers = dict(headers)

        if self._access_token:
            headers["authorization"] = self._access_token

        response = await self._session.request(
            method,
            f"{self._base_url}/{path}",
            **kwargs,
            headers=headers,
        )
        response.raise_for_status()
        return response

    async def get(self, path: str) -> ClientResponse:
        """Send a GET request."""
        return await self.request("GET", path)

    async def put(self, path: str, **kwargs: dict[str, Any]) -> ClientResponse:
        """Send a PUT request."""
        return await self.request("PUT", path, **kwargs)

    async def post(self, path: str, **kwargs: dict[str, Any]) -> ClientResponse:
        """Send a POST request."""
        return await self.request("POST", path, **kwargs)

    async def delete(self, path: str) -> ClientResponse:
        """Send a DELETE request."""
        return await self.request("DELETE", path)



from enum import IntEnum
from typing import Generic, TypeVar

from pydantic import BaseModel, ConfigDict, Field, NonNegativeFloat

StatusT = TypeVar("StatusT")
ColorT = TypeVar("ColorT")


class Status(BaseModel, Generic[StatusT]):
    status: StatusT


class FadeTime(BaseModel):
    fade_time: NonNegativeFloat = Field(alias="fadeTime")


class DimmableWithFadeTime(FadeTime):
    """Data model for the ``dimmableWithFade`` feature."""

    dim_value: NonNegativeFloat = Field(alias="dimValue", ge=0.0, le=100.0)


class ColorWithFadeTime(FadeTime, Generic[ColorT]):
    """Data model for color features with fade time."""

    color: ColorT


class ColorRGBData(BaseModel):
    """Data model for the ``colorRGB`` feature."""

    red: float | None = Field(
        None,
        description="Relative red value in the [0, 1] interval.",
        alias="r",
        ge=0.0,
        le=1.0,
    )
    green: float | None = Field(
        None,
        description="Relative green value in the [0, 1] interval.",
        alias="g",
        ge=0.0,
        le=1.0,
    )
    blue: float | None = Field(
        None,
        description="Relative blue value in the [0, 1] interval.",
        alias="b",
        ge=0.0,
        le=1.0,
    )

    model_config = ConfigDict(
        json_schema_extra={"examples": [{"r": 0.0, "g": 0.5, "b": 1.0}]}
    )


class DimmableRGBData(ColorRGBData):
    """Data model for the ``dimmableRGB`` feature."""

    dimmable: float = Field(
        50.0,
        description="Percentage in the [0, 100] interval to dim a device to.",
        ge=0.0,
        le=100.0,
    )


class ColorWAFData(BaseModel):
    """Data model for the ``colorWAF`` feature."""

    white: float | None = Field(
        None,
        description="Relative white value in the [0, 1] interval.",
        alias="w",
        ge=0.0,
        le=1.0,
    )
    amber: float | None = Field(
        None,
        description="Relative amber value in the [0, 1] interval.",
        alias="a",
        ge=0.0,
        le=1.0,
    )
    free_color: float | None = Field(
        None,
        description="Relative free color value in the [0, 1] interval.",
        alias="f",
        ge=0.0,
        le=1.0,
    )

    model_config = ConfigDict(
        json_schema_extra={"examples": [{"w": 0.0, "a": 0.5, "f": 1.0}]}
    )


class DimmableWAFData(ColorWAFData):
    """Data model for the ``dimmableWAF`` feature."""

    dimmable: float = Field(
        50.0,
        description="Percentage in the [0, 100] interval to dim a device to.",
        ge=0.0,
        le=100.0,
    )


class DimmableKelvinData(BaseModel):
    """Data model for the ``dimmableKelvin`` feature."""

    dimmable: float = Field(
        50.0,
        description="Percentage in the [0, 100] interval to dim a device to.",
        ge=0.0,
        le=100.0,
    )
    kelvin: NonNegativeFloat = Field(4000.0, description="Color temperature in Kelvin.")


class ColorXYData(BaseModel):
    """Data model for the ``colorXY`` feature."""

    x: float | None = Field(
        None,
        description="X coordinate in the CIE color chromaticity space.",
    )
    y: float | None = Field(
        None,
        description="Y coordinate in the CIE color chromaticity space.",
    )

    model_config = ConfigDict(
        json_schema_extra={"examples": [{"x": 0.432, "y": 0.150}]}
    )


class DimmableXYData(ColorXYData):
    """Data model for the ``dimmableXY`` feature."""

    dimmable: float = Field(
        50.0,
        description="Percentage in the [0, 100] interval to dim a device to.",
        ge=0.0,
        le=100.0,
    )


class TimeSignature(BaseModel):
    timestamp: float
    counter: int


class FeaturesStatus(BaseModel):
    """Data model for the features status."""

    switchable: Status[bool] | None = None
    dimmable: Status[float] | None = None
    dimmable_with_fade: Status[DimmableWithFadeTime] | None = Field(
        None, alias="dimmableWithFade"
    )
    dim_up: bool | None = Field(None, alias="dimUp")
    dim_down: bool | None = Field(None, alias="dimDown")
    scene: bool | None = None
    scene_with_fade: Status[FadeTime] | None = Field(None, alias="sceneWithFade")
    goto_last_active: dict | None = Field(None, alias="gotoLastActive")
    goto_last_active_with_fade: Status[FadeTime] | None = Field(
        None, alias="gotoLastActiveWithFade"
    )
    dali_cmd16: bool | None = Field(None, alias="daliCmd16")
    fade_time: Status[float] | None = Field(None, alias="fadeTime")
    fade_rate: Status[float] | None = Field(None, alias="fadeRate")
    save_to_scene: bool | None = Field(None, alias="saveToScene")
    color_rgb: Status[ColorRGBData] | None = Field(None, alias="colorRGB")
    dimmable_rgb: Status[DimmableRGBData] | None = Field(None, alias="dimmableRGB")
    color_rgb_with_fade: Status[ColorWithFadeTime[ColorRGBData]] | None = Field(
        None, alias="colorRGBWithFade"
    )
    color_waf: Status[ColorWAFData] | None = Field(None, alias="colorWAF")
    dimmable_waf: Status[DimmableWAFData] | None = Field(None, alias="dimmableWAF")
    color_waf_with_fade: Status[ColorWithFadeTime[ColorWAFData]] | None = Field(
        None, alias="colorWAFWithFade"
    )
    color_kelvin: Status[NonNegativeFloat] | None = Field(None, alias="colorKelvin")
    dimmable_kelvin: Status[DimmableKelvinData] | None = Field(
        None, alias="dimmableKelvin"
    )
    color_kelvin_with_fade: Status[ColorWithFadeTime[NonNegativeFloat]] | None = Field(
        None, alias="colorKelvinWithFade"
    )
    color_xy: Status[ColorXYData] | None = Field(None, alias="colorXY")
    dimmable_xy: Status[DimmableXYData] | None = Field(None, alias="dimmableXY")
    color_xy_with_fade: Status[ColorWithFadeTime[ColorXYData | dict]] | None = Field(
        None, alias="colorXYWithFade"
    )


class DALIType(IntEnum):
    """DALI device types."""

    DT0_FLUORESCENT_LAMPS = 0
    DT1_EMERGENCY_LIGHTING = 1
    DT2_DISCHARGE_LAMPS = 2
    DT3_LOW_VOLTAGE_HALOGEN_LAMPS = 3
    DT4_SUPPLY_VOLTAGE_CONTROLLER = 4
    DT5_CONVERSION_FROM_DIGITAL_SIGNAL_INTO_DC_VOLTAGE = 5
    DT6_LED_MODULES = 6
    DT7_SWITCHING_FUNCTION = 7
    DT8_COLOUR_CONTROL = 8
    DT9_SEQUENCER = 9



from typing import ClassVar, Final

from lunatone_rest_api_client import Auth
from lunatone_rest_api_client.base import Controllable
from lunatone_rest_api_client.models import (
    ControlData,
    ZoneData,
    ZonesData,
)

_PATH: Final[str] = "zones"


class Zone(Controllable):
    """Class that represents a zone object in the API."""

    base_path: ClassVar[str] = _PATH[:-1]

    def __init__(self, auth: Auth, data: ZoneData) -> None:
        """Initialize a zone object."""
        self._auth = auth
        self._data = data

    @property
    def path(self) -> str:
        """Return the resource path."""
        return f"{self.base_path}/{self.id}"

    @property
    def data(self) -> ZoneData:
        """Return the raw zone data."""
        return self._data

    @property
    def id(self) -> int:
        """Return the ID of the zone."""
        return self.data.id

    @property
    def name(self) -> str:
        """Return the name of the zone."""
        return self.data.name

    async def async_update(self) -> None:
        """Update the zone data."""
        response = await self._auth.get(self.path)
        self._data = ZoneData.model_validate(await response.json())

    async def async_control(self, data: ControlData) -> None:
        """Control the zone."""
        json_data = data.model_dump(by_alias=True, exclude_none=True)
        await self._auth.post(f"{self.path}/control", json=json_data)


class Zones:
    """Class that represents a zones object in the API."""

    path: ClassVar[str] = _PATH

    def __init__(self, auth: Auth) -> None:
        """Initialize a zones object."""
        self._auth = auth
        self._data: ZonesData | None = None

    @property
    def data(self) -> ZonesData | None:
        """Return the raw zones data."""
        return self._data

    @property
    def zones(self) -> list[Zone]:
        """Return a list of zones."""
        return [Zone(self._auth, zone) for zone in self.data.zones] if self.data else []

    async def async_update(self) -> None:
        """Update the zones data."""
        response = await self._auth.get(self.path)
        self._data = ZonesData.model_validate(await response.json())



import aiohttp
import pytest
from aioresponses.core import aioresponses

from lunatone_rest_api_client import Auth, Info
from lunatone_rest_api_client.info import (
    ARTICLE_INFO_NAME_MAPPING,
    ARTICLE_NUMBER_NAME_MAPPING,
)
from lunatone_rest_api_client.models import InfoData

from .common import INFO_DATA_RAW


@pytest.fixture
def info_data() -> InfoData:
    return InfoData.model_validate(INFO_DATA_RAW)


@pytest.mark.asyncio
async def test_info_update(
    aioresponses: aioresponses, base_url: str, info_data: InfoData
) -> None:
    json_data = info_data.model_dump(by_alias=True)
    aioresponses.get(f"{base_url}/{Info.path}", payload=json_data)

    async with aiohttp.ClientSession() as session:
        info = Info(Auth(session, base_url))
        await info.async_update()
        assert info.data == info_data


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "article_number, article_info",
    [
        (86456840, "-NR"),
        (86456841, ""),
        (89453886, ""),
        (22176625, ""),
        (22176625, "-PS-NR"),
    ],
)
async def test_product_name(
    aioresponses: aioresponses,
    base_url: str,
    info_data: InfoData,
    article_number: int,
    article_info: str,
) -> None:
    info_data.device.article_number = article_number
    info_data.device.article_info = article_info

    json_data = info_data.model_dump(by_alias=True)
    aioresponses.get(f"{base_url}/{Info.path}", payload=json_data)

    async with aiohttp.ClientSession() as session:
        info = Info(Auth(session, base_url))
        await info.async_update()
        assert info.data == info_data

        expected_product_name = ARTICLE_NUMBER_NAME_MAPPING[article_number]
        if info.article_info:
            for key, value in ARTICLE_INFO_NAME_MAPPING.items():
                if key in info.article_info:
                    expected_product_name += f" {value}"
        assert expected_product_name == info.product_name



from enum import StrEnum

from pydantic import BaseModel, Field


class DescriptorData(BaseModel):
    lines: int = 0
    buffer_size: int = Field(0, alias="bufferSize")
    tick_resolution: int = Field(0, alias="tickResolution")
    max_yn_frame_size: int = Field(0, alias="maxYnFrameSize")
    implemented_macros: list[int] = Field([], alias="implementedMacros")
    device_list_specifier: int = Field(0, alias="deviceListSpecifier")
    protocol_version_major: int = Field(1, alias="protocolVersionMajor")
    protocol_version_minor: int = Field(0, alias="protocolVersionMinor")
    power_supply_implemented: bool = Field(False, alias="powerSupplyImplemented")


class DeviceInfoData(BaseModel):
    serial: int
    gtin: int
    pcb: str
    article_number: int = Field(alias="articleNumber")
    article_info: str = Field("", alias="articleInfo")
    production_year: int = Field(alias="productionYear")
    production_week: int = Field(alias="productionWeek")


class LineStatus(StrEnum):
    OK = "ok"
    LOW_POWER = "lowPower"
    NO_POWER = "noPower"
    NOT_REACHABLE = "notReachable"


class DALIBusData(BaseModel):
    send_blocked_initialize: bool = Field(False, alias="sendBlockedInitialize")
    send_blocked_quiescent: bool = Field(False, alias="sendBlockedQuiescent")
    send_blocked_macro_running: bool = Field(False, alias="sendBlockedMacroRunning")
    send_buffer_full: bool = Field(False, alias="sendBufferFull")
    line_status: LineStatus = Field(LineStatus.OK, alias="lineStatus")
    device: DeviceInfoData

    @property
    def is_line_status_ok(self) -> bool:
        """Return if the line status is OK."""
        return self.line_status == LineStatus.OK


class StartupMode(StrEnum):
    NORMAL = "normal"
    MINIMAL = "minimal"


class InfoData(BaseModel):
    name: str
    version: str
    tier: str = "basic"
    emergency_light: bool = Field(False, alias="emergencyLight")
    node_red: bool = Field(False, alias="nodeRed")
    startup_mode: StartupMode | None = Field(StartupMode.NORMAL, alias="startupMode")
    errors: dict[str, str] = {}
    descriptor: DescriptorData = DescriptorData()
    device: DeviceInfoData
    lines: dict[str, DALIBusData] = {}



"""Discovery module for Lunatone interfaces."""

import asyncio
import json
import socket
from dataclasses import dataclass
from enum import IntFlag, StrEnum, auto
from typing import Any, AsyncGenerator, Final

DISCOVERY_ADDRESS: Final[str] = "255.255.255.255"
DISCOVERY_PORT: Final[int] = 5555
DISCOVERY_MESSAGE: Final[str] = "discovery"
DISCOVERY_LISTEN_TIMEOUT_SECONDS: Final[float] = 3.0


class LunatoneDiscoveryType(StrEnum):
    """Discovery types."""

    DALI2_IOT = "dali-2-iot"
    DALI2_IOT4 = "dali-2-iot4"
    DALI2_DISPLAY = "dali-2-display"
    DALI_WIFI = "dali-wifi"
    AIR_WLAN_SWITCH = "air-wlan-switch"


class Feature(IntFlag):
    """Features."""

    REST_API = auto()


SUPPORTED_FEATURES: dict[LunatoneDiscoveryType, set[Feature]] = {
    LunatoneDiscoveryType.DALI2_IOT: (Feature.REST_API),
    LunatoneDiscoveryType.DALI2_IOT4: (Feature.REST_API),
    LunatoneDiscoveryType.DALI2_DISPLAY: (Feature.REST_API),
    LunatoneDiscoveryType.DALI_WIFI: (),
    LunatoneDiscoveryType.AIR_WLAN_SWITCH: (),
}


@dataclass
class LunatoneDiscoveryInfo:
    """LunatoneDiscoveryInfo."""

    host: str
    name: str
    type: LunatoneDiscoveryType


class LunatoneUDPProtocol(asyncio.DatagramProtocol):
    """UDP protocol implementation for discovering Lunatone interfaces."""

    def __init__(self, queue: asyncio.Queue) -> None:
        """Initialize the UDP protocol instance."""
        super().__init__()
        self._queue: asyncio.Queue[LunatoneDiscoveryInfo] = queue

    def datagram_received(self, data: bytes, addr: tuple[str | Any, int]) -> None:
        """Handle an incoming UDP datagram."""
        try:
            message = json.loads(data.decode())
        except json.JSONDecodeError:
            return
        if not ("type" in message and "name" in message):  # Validate message
            return
        if message["type"] in LunatoneDiscoveryType:
            info = LunatoneDiscoveryInfo(addr[0], message["name"], message["type"])
            self._queue.put_nowait(info)


async def async_discover_devices_stream(
    loop: asyncio.AbstractEventLoop,
    timeout: float = DISCOVERY_LISTEN_TIMEOUT_SECONDS,
    local_ip: str | None = None,
    filter_feature: Feature | None = None,
) -> AsyncGenerator[LunatoneDiscoveryInfo, None]:
    """Yield devices one by one as they arrive (streaming style)."""
    queue: asyncio.Queue[LunatoneDiscoveryInfo] = asyncio.Queue()
    transport, _ = await loop.create_datagram_endpoint(
        lambda: LunatoneUDPProtocol(queue),
        local_addr=(local_ip or "0.0.0.0", DISCOVERY_PORT),
        family=socket.AF_INET,
        allow_broadcast=True,
    )
    try:
        transport.sendto(DISCOVERY_MESSAGE.encode(), (DISCOVERY_ADDRESS, DISCOVERY_PORT))
        end = loop.time() + timeout
        while loop.time() < end:
            try:
                device = await asyncio.wait_for(queue.get(), timeout=end - loop.time())
                if (
                    filter_feature is None
                    or filter_feature in SUPPORTED_FEATURES[device.type]
                ):
                    yield device
            except KeyError:
                continue
            except asyncio.TimeoutError:
                break
    finally:
        transport.close()


async def async_discover_devices(
    loop: asyncio.AbstractEventLoop,
    timeout: float = DISCOVERY_LISTEN_TIMEOUT_SECONDS,
    local_ip: str = "0.0.0.0",
    filter_features: int = Feature.REST_API,
) -> list[LunatoneDiscoveryInfo]:
    """Collect all devices and return them at once (batch style)."""
    devices: list[LunatoneDiscoveryInfo] = []
    async for dev in async_discover_devices_stream(
        loop, timeout, local_ip, filter_features
    ):
        devices.append(dev)
    return devices



from typing import Any

INFO_DATA_RAW: dict[str, Any] = {
    "name": "DALI-2 Display",
    "version": "v1.14.1/1.4.3",
    "tier": "plus",
    "emergencyLight": False,
    "nodeRed": False,
    "errors": {},
    "descriptor": {
        "lines": 1,
        "bufferSize": 32,
        "tickResolution": 1978,
        "maxYnFrameSize": 32,
        "implementedMacros": [
            132,
            134,
            138,
            140,
            144,
            146,
            148,
            150,
            158,
            164,
            184,
            186,
        ],
        "deviceListSpecifier": 1,
        "protocolVersionMajor": 1,
        "protocolVersionMinor": 6,
        "powerSupplyImplemented": False,
    },
    "device": {
        "serial": 1234,
        "gtin": 9010342013577,
        "pcb": "9a",
        "articleNumber": 86456841,
        "articleInfo": "",
        "productionYear": 2021,
        "productionWeek": 31,
        "deviceType": 67,
    },
    "lines": {
        "0": {
            "sendBlockedInitialize": False,
            "sendBlockedQuiescent": False,
            "sendBlockedMacroRunning": False,
            "sendBufferFull": False,
            "lineStatus": "ok",
            "device": {
                "serial": 1234,
                "gtin": 9010342013577,
                "pcb": "9a",
                "articleNumber": 86456841,
                "articleInfo": "",
                "productionYear": 2021,
                "productionWeek": 31,
                "deviceType": 67,
            },
        }
    },
}
DEVICES_DATA_RAW: dict[str, Any] = {
    "devices": [
        {
            "id": 1,
            "name": "Test Device 1",
            "type": "default",
            "features": {
                "switchable": {"status": True},
                "dimmable": {"status": 0},
                "dimmableWithFade": {"status": {"dimValue": 0, "fadeTime": 1}},
                "scene": True,
                "sceneWithFade": {"status": {"fadeTime": 1}},
                "gotoLastActive": {},
                "gotoLastActiveWithFade": {"status": {"fadeTime": 1}},
                "daliCmd16": True,
                "fadeTime": {"status": 1},
                "fadeRate": {"status": 63.3},
                "saveToScene": True,
                "colorRGB": {"status": {"r": 1, "g": 1, "b": 1}},
                "colorRGBWithFade": {
                    "status": {"color": {"r": 1, "g": 1, "b": 1}, "fadeTime": 1}
                },
                "colorWAF": {"status": {"w": 1, "a": 1, "f": 1}},
                "colorWAFWithFade": {
                    "status": {"color": {"w": 1, "a": 1, "f": 1}, "fadeTime": 1}
                },
                "colorKelvin": {"status": 2700},
                "colorKelvinWithFade": {"status": {"color": 2700, "fadeTime": 1}},
                "colorXY": {"status": {"x": 0, "y": 0}},
                "colorXYWithFade": {
                    "status": {"color": {"x": 0, "y": 0}, "fadeTime": 1}
                },
            },
            "scenes": [],
            "groups": [],
            "address": 0,
            "line": 0,
            "daliTypes": [],
        },
        {
            "id": 2,
            "name": "Test Device 2",
            "type": "default",
            "features": {
                "switchable": {"status": False},
                "dimmable": {"status": 0},
                "dimmableWithFade": {"status": {"dimValue": 0, "fadeTime": 1}},
                "scene": True,
                "sceneWithFade": {"status": {"fadeTime": 1}},
                "gotoLastActive": {},
                "gotoLastActiveWithFade": {"status": {"fadeTime": 1}},
                "daliCmd16": True,
                "fadeTime": {"status": 1},
                "fadeRate": {"status": 63.3},
                "saveToScene": True,
                "colorRGB": {"status": {"r": 1, "g": 1, "b": 1}},
                "colorRGBWithFade": {
                    "status": {"color": {"r": 1, "g": 1, "b": 1}, "fadeTime": 1}
                },
                "colorWAF": {"status": {"w": 1, "a": 1, "f": 1}},
                "colorWAFWithFade": {
                    "status": {"color": {"w": 1, "a": 1, "f": 1}, "fadeTime": 1}
                },
                "colorKelvin": {"status": 2700},
                "colorKelvinWithFade": {"status": {"color": 2700, "fadeTime": 1}},
                "colorXY": {"status": {"x": 0, "y": 0}},
                "colorXYWithFade": {
                    "status": {"color": {"x": 0, "y": 0}, "fadeTime": 1}
                },
            },
            "scenes": [],
            "groups": [],
            "address": 1,
            "line": 0,
            "daliTypes": [],
        },
    ],
    "timeSignature": {"timestamp": 946688334.0687395, "counter": 2028},
}
SENSORS_DATA_RAW: dict[str, Any] = {
    "sensors": [
        {
            "id": 0,
            "name": "Temperature Sensor",
            "unit": "°C",
            "type": "temperature",
            "value": 26.5,
            "timestamp": "2000-01-01T05:32:08.487680",
            "addressType": "internal",
            "daliSensorAddress": None,
        },
        {
            "id": 1,
            "name": "CPU temperature",
            "unit": "°C",
            "type": "temperature",
            "value": 0,
            "timestamp": "0001-01-01T00:00:00",
            "addressType": "internal",
            "daliSensorAddress": None,
        },
        {
            "id": 2,
            "name": "Air humidity",
            "unit": "%",
            "type": "airHumidity",
            "value": 66,
            "timestamp": "2000-01-01T05:18:32.452098",
            "addressType": "internal",
            "daliSensorAddress": None,
        },
        {
            "id": 3,
            "name": "Air pressure",
            "unit": "hPa",
            "type": "airPressure",
            "value": 986.5,
            "timestamp": "2000-01-01T05:31:32.673799",
            "addressType": "internal",
            "daliSensorAddress": None,
        },
        {
            "id": 4,
            "name": "eCO2",
            "unit": "ppm",
            "type": "eCO2",
            "value": 3281.1000000000004,
            "timestamp": "2000-01-01T05:33:56.684759",
            "addressType": "internal",
            "daliSensorAddress": None,
        },
        {
            "id": 5,
            "name": "VOC",
            "unit": "ppb",
            "type": "VOC",
            "value": 0,
            "timestamp": "0001-01-01T00:00:00",
            "addressType": "internal",
            "daliSensorAddress": None,
        },
        {
            "id": 6,
            "name": "Air quality",
            "unit": "AQI",
            "type": "airQuality",
            "value": 328,
            "timestamp": "2000-01-01T05:33:56.611870",
            "addressType": "internal",
            "daliSensorAddress": None,
        },
    ]
}
ZONES_DATA_RAW = {
    "zones": [
        {
            "id": 1,
            "name": "DALI broadcast on lines 0-3",
            "targets": [
                {"type": "broadcast", "id": 0},
                {"type": "broadcast", "id": 1},
                {"type": "broadcast", "id": 2},
                {"type": "broadcast", "id": 3},
            ],
            "features": {
                "switchable": {"status": False},
                "dimmable": {"status": 0},
                "dimUp": True,
                "dimDown": True,
                "scene": True,
                "gotoLastActive": {},
                "daliCmd16": True,
                "saveToScene": True,
                "colorRGB": {"status": {"r": 1, "g": 1, "b": 1}},
                "dimmableRGB": {"status": {"r": 1, "g": 1, "b": 1, "dimmable": 0}},
                "colorWAF": {"status": {"w": 1, "a": 1, "f": 1}},
                "dimmableWAF": {"status": {"w": 1, "a": 1, "f": 1, "dimmable": 0}},
                "colorXY": {"status": {"x": 1, "y": 1}},
                "dimmableXY": {"status": {"x": 1, "y": 1, "dimmable": 0}},
            },
        },
        {
            "id": 2,
            "name": "group 0",
            "targets": [{"type": "group", "id": 0}],
            "features": {
                "switchable": {"status": False},
                "dimmable": {"status": 0},
                "dimmableWithFade": {"status": {"dimValue": 0, "fadeTime": 1}},
                "dimUp": True,
                "dimDown": True,
                "scene": True,
                "sceneWithFade": {"status": {"fadeTime": 1}},
                "gotoLastActive": {},
                "gotoLastActiveWithFade": {"status": {"fadeTime": 1}},
                "daliCmd16": True,
                "fadeTime": {"status": 1},
                "fadeRate": {"status": 22.4},
                "saveToScene": True,
                "colorRGB": {"status": {"r": 1, "g": 1, "b": 1}},
                "dimmableRGB": {"status": {"r": 1, "g": 1, "b": 1, "dimmable": 0}},
                "colorRGBWithFade": {
                    "status": {"color": {"r": 1, "g": 1, "b": 1}, "fadeTime": 1}
                },
                "colorWAF": {"status": {"w": 1, "a": 1, "f": 1}},
                "dimmableWAF": {"status": {"w": 1, "a": 1, "f": 1, "dimmable": 0}},
                "colorWAFWithFade": {
                    "status": {"color": {"w": 1, "a": 1, "f": 1}, "fadeTime": 1}
                },
                "colorKelvin": {"status": 4000},
                "dimmableKelvin": {"status": {"dimmable": 0, "kelvin": 4000}},
                "colorKelvinWithFade": {"status": {"color": 4000, "fadeTime": 1}},
                "colorXY": {"status": {"x": 1, "y": 1}},
                "dimmableXY": {"status": {"x": 1, "y": 1, "dimmable": 0}},
                "colorXYWithFade": {
                    "status": {"color": {"x": 1, "y": 1}, "fadeTime": 1}
                },
            },
        },
        {
            "id": 3,
            "name": "group 1",
            "targets": [{"type": "group", "id": 1}],
            "features": {
                "switchable": {"status": False},
                "dimmable": {"status": 0},
                "dimUp": True,
                "dimDown": True,
                "scene": True,
                "gotoLastActive": {},
                "daliCmd16": True,
                "saveToScene": True,
                "colorRGB": {"status": {"r": 1, "g": 1, "b": 1}},
                "dimmableRGB": {"status": {"r": 1, "g": 1, "b": 1, "dimmable": 0}},
                "colorWAF": {"status": {"w": 1, "a": 1, "f": 1}},
                "dimmableWAF": {"status": {"w": 1, "a": 1, "f": 1, "dimmable": 0}},
                "colorKelvin": {"status": 2700},
                "dimmableKelvin": {"status": {"dimmable": 0, "kelvin": 2700}},
                "colorXY": {"status": {"x": 1, "y": 1}},
                "dimmableXY": {"status": {"x": 1, "y": 1, "dimmable": 0}},
            },
        },
        {
            "id": 4,
            "name": "group 2",
            "targets": [{"type": "group", "id": 2}],
            "features": {
                "switchable": {"status": False},
                "dimmable": {"status": 0},
                "dimmableWithFade": {"status": {"dimValue": 0, "fadeTime": 0.7}},
                "dimUp": True,
                "dimDown": True,
                "scene": True,
                "sceneWithFade": {"status": {"fadeTime": 0.7}},
                "gotoLastActive": {},
                "gotoLastActiveWithFade": {"status": {"fadeTime": 0.7}},
                "daliCmd16": True,
                "fadeTime": {"status": 0.7},
                "fadeRate": {"status": 44.7},
                "saveToScene": True,
                "colorRGB": {"status": {"r": 1, "g": 1, "b": 1}},
                "dimmableRGB": {"status": {"r": 1, "g": 1, "b": 1, "dimmable": 0}},
                "colorRGBWithFade": {
                    "status": {"color": {"r": 1, "g": 1, "b": 1}, "fadeTime": 0.7}
                },
                "colorWAF": {"status": {"w": 1, "a": 1, "f": 1}},
                "dimmableWAF": {"status": {"w": 1, "a": 1, "f": 1, "dimmable": 0}},
                "colorWAFWithFade": {
                    "status": {"color": {"w": 1, "a": 1, "f": 1}, "fadeTime": 0.7}
                },
                "colorKelvin": {"status": 2700},
                "dimmableKelvin": {"status": {"dimmable": 0, "kelvin": 2700}},
                "colorKelvinWithFade": {"status": {"color": 2700, "fadeTime": 0.7}},
                "colorXY": {"status": {"x": 1, "y": 1}},
                "dimmableXY": {"status": {"x": 1, "y": 1, "dimmable": 0}},
                "colorXYWithFade": {
                    "status": {"color": {"x": 1, "y": 1}, "fadeTime": 0.7}
                },
            },
        },
    ],
    "timeSignature": {"timestamp": 946712827.9615544, "counter": 3494},
}



import aiohttp
import pytest
from aioresponses.core import aioresponses

from lunatone_rest_api_client import Auth, Zone, Zones
from lunatone_rest_api_client.models import ZoneData, ZonesData

from .common import ZONES_DATA_RAW


@pytest.fixture
def zones_data() -> ZonesData:
    return ZonesData.model_validate(ZONES_DATA_RAW)


@pytest.fixture
def zone_data(zones_data: ZonesData) -> ZoneData:
    for zone_data in zones_data.zones:
        return zone_data


@pytest.mark.asyncio
async def test_zones_update(
    aioresponses: aioresponses, base_url: str, zones_data: ZonesData
) -> None:
    json_data = zones_data.model_dump(by_alias=True)
    aioresponses.get(f"{base_url}/{Zones.path}", payload=json_data)

    async with aiohttp.ClientSession() as session:
        zones = Zones(Auth(session, base_url))
        await zones.async_update()
        assert zones.data == zones_data


@pytest.mark.asyncio
async def test_zone_update(
    aioresponses: aioresponses, base_url: str, zone_data: ZoneData
) -> None:
    json_data = zone_data.model_dump(by_alias=True)
    aioresponses.get(f"{base_url}/{Zone.base_path}/{zone_data.id}", payload=json_data)

    async with aiohttp.ClientSession() as session:
        zones = Zone(Auth(session, base_url), ZoneData(id=zone_data.id))
        await zones.async_update()
        assert zones.data == zone_data



import argparse
import asyncio

import aiohttp

from lunatone_rest_api_client import (
    Auth,
    Device,
    Devices,
)
from lunatone_rest_api_client.models import ControlData


parser = argparse.ArgumentParser(prog="API Test")
parser.add_argument("-i", "--ip", type=str, required=True,
                    help="Specify IP address of the device")
arguments = parser.parse_args()


async def test_consecutively_get_and_post_requests(auth: Auth):
    devices = Devices(auth)
    await devices.async_update() # Send GET request to /devices

    device: Device = devices.devices[0]

    # Send POST request with { "switchable": true } to /device/1/control
    await device.async_control(ControlData(switchable=True))
    await device.async_update() # Send GET request to /device/1
    print(f"request1: expected switchable=True, actual switchable={device.data.features.switchable}")
    await device.async_update() # Send GET request to /device/1
    print(f"request2: expected switchable=True, actual switchable={device.data.features.switchable}")

    # Send POST request with { "switchable": false } to /device/1/control
    await device.async_control(ControlData(switchable=False))
    await device.async_update() # Send GET request to /device/1
    print(f"request1: expected switchable=False, actual switchable={device.data.features.switchable}")
    await device.async_update() # Send GET request to /device/1
    print(f"request2: expected switchable=False, actual switchable={device.data.features.switchable}")


async def main():
    async with aiohttp.ClientSession() as session:
        auth = Auth(session, f"http://{arguments.ip}")

        await test_consecutively_get_and_post_requests(auth)


if __name__ == "__main__":
    asyncio.run(main())



from typing import ClassVar, Final

from lunatone_rest_api_client import Auth
from lunatone_rest_api_client.models import SensorData, SensorsData

_PATH: Final[str] = "sensors"


class Sensor:
    """Class that represents a Sensor object in the API."""

    base_path: ClassVar[str] = _PATH

    def __init__(self, auth: Auth, data: SensorData) -> None:
        """Initialize a Sensor object."""
        self._auth = auth
        self._data = data

    @property
    def path(self) -> str:
        """Return the resource path."""
        return f"{self.base_path}/{self.id}"

    @property
    def data(self) -> SensorData:
        """Return the raw sensor data."""
        return self._data

    @property
    def id(self) -> int:
        """Return the ID of the zone."""
        return self.data.id

    @property
    def name(self) -> str:
        """Return the name of the zone."""
        return self.data.name

    async def async_update(self) -> None:
        """Update the sensor data."""
        response = await self._auth.get(self.path)
        self._data = SensorData.model_validate(await response.json())

    async def async_refresh(self) -> None:
        """Refresh the sensor data by reading it from the DALI bus."""
        response = await self._auth.post(self.path)
        self._data = SensorData.model_validate(await response.json())


class Sensors:
    """Class that represents a Sensors object in the API."""

    path: ClassVar[str] = _PATH

    def __init__(self, auth: Auth) -> None:
        """Initialize a Sensors object."""
        self._auth = auth
        self._data: SensorsData | None = None

    @property
    def data(self) -> SensorsData | None:
        """Return the raw sensors data."""
        return self._data

    @property
    def sensors(self) -> list[Sensor]:
        """Return a list of sensors."""
        return (
            [Sensor(self._auth, sensor) for sensor in self.data.sensors]
            if self.data
            else []
        )

    async def async_update(self) -> None:
        """Update the sensors data."""
        response = await self._auth.get(self.path)
        self._data = SensorsData.model_validate(await response.json())

    async def async_refresh(self) -> None:
        """Refresh the sensors data by reading it from the DALI bus."""
        response = await self._auth.post(self.path)
        self._data = SensorsData.model_validate(await response.json())



from enum import StrEnum

from pydantic import BaseModel, Field, NonNegativeInt


class StartScanData(BaseModel):
    new_installation: bool = Field(False, alias="newInstallation")
    no_addressing: bool = Field(False, alias="noAddressing")
    use_lines: list[NonNegativeInt] = Field([], alias="useLines")


class ScanState(StrEnum):
    NOT_STARTED = "not started"
    CANCELLED = "cancelled"
    DONE = "done"
    ADDRESSING = "addressing"
    IN_PROGRESS = "in progress"


class ScanData(BaseModel):
    id: str = ""
    progress: float | None = None
    found: int | None = None
    found_sensors: int | None = Field(None, alias="foundSensors")
    status: ScanState = ScanState.NOT_STARTED
    lines: list[dict] = []



from typing import ClassVar

from lunatone_rest_api_client import Auth
from lunatone_rest_api_client.models import InfoData


ARTICLE_NUMBER_NAME_MAPPING: dict[int, str] = {
    86456840: "DALI-2 Display 7''",
    86456841: "DALI-2 Display 4''",
    89453886: "DALI-2 IoT",
    22176625: "DALI-2 IoT4",
}

ARTICLE_INFO_NAME_MAPPING: dict[str, str] = {
    "NR": "Node-RED",
    "PS": "integrated power supply",
}


class Info:
    """Class that represents a info object in the API."""

    path: ClassVar[str] = "info"

    def __init__(self, auth: Auth) -> None:
        """Initialize an info object."""
        self._auth = auth
        self._data: InfoData | None = None

    @property
    def data(self) -> InfoData | None:
        """Return the raw info data."""
        return self._data

    @property
    def name(self) -> str | None:
        """Return the name of the API interface."""
        return self.data.name if self.data else None

    @property
    def version(self) -> str | None:
        """Return the software version of the API interface."""
        return self.data.version if self.data else None

    @property
    def article_number(self) -> int | None:
        """Return the article number of the API interface."""
        return self.data.device.article_number if self.data else None

    @property
    def article_info(self) -> str | None:
        """Return the article info of the API interface."""
        return self.data.device.article_info if self.data else None

    @property
    def serial_number(self) -> int | None:
        """Return the serial number of the API interface."""
        return self.data.device.serial if self.data else None

    @property
    def product_name(self) -> str | None:
        """Return the product name of the API interface."""
        name = ARTICLE_NUMBER_NAME_MAPPING.get(self.article_number, None)
        if name is not None:
            for key, value in ARTICLE_INFO_NAME_MAPPING.items():
                if self.article_info is not None and self.article_info.find(key) > 0:
                    name += f" {value}"
        return name

    @property
    def is_line_available(self, line: int | None = None) -> bool:
        """Return if a specific line or if none is provided any line is available."""
        if line is None:
            return any(l.is_line_status_ok for l in self.data.lines)
        else:
            return self.data.lines[str(line)].is_line_status_ok

    async def async_update(self) -> None:
        """Update the info data."""
        response = await self._auth.get(self.path)
        self._data = InfoData.model_validate(await response.json())



from pydantic import BaseModel, ConfigDict, Field, NonNegativeFloat, NonNegativeInt

from lunatone_rest_api_client.models.common import (
    ColorRGBData,
    ColorWAFData,
    ColorWithFadeTime,
    ColorXYData,
    DimmableKelvinData,
    DimmableRGBData,
    DimmableWAFData,
    DimmableWithFadeTime,
    DimmableXYData,
    FadeTime,
)


class SceneWithFadeData(FadeTime):
    scene: int = Field(ge=0, lt=16)


class ControlData(BaseModel):
    """Data model for ``ControlData``."""

    switchable: bool | None = None
    dimmable: float | None = Field(None, ge=0.0, le=100.0)
    dimmable_with_fade: DimmableWithFadeTime | None = Field(
        None, alias="dimmableWithFade"
    )
    dim_up: int | None = Field(None, alias="dimUp", ge=1, le=1)
    dim_down: int | None = Field(None, alias="dimDown", ge=1, le=1)
    goto_last_active: bool | None = Field(
        None,
        description="Value must be ``true``.",
        alias="gotoLastActive",
    )
    goto_last_active_with_fade: FadeTime | None = Field(
        None,
        description="Dim to the last level within a fade time in seconds.",
        alias="gotoLastActiveWithFade",
    )
    scene: int | None = Field(
        None,
        description="Scene number of the scene to recall.",
        ge=0,
        lt=16,
    )
    scene_with_fade: SceneWithFadeData | None = Field(
        None,
        description="Scene number of the scene to recall, within a fade time in seconds.",
        alias="sceneWithFade",
    )
    fade_time: NonNegativeFloat | None = Field(
        None,
        description="Set the fade time in seconds.",
        alias="fadeTime",
    )
    fade_rate: NonNegativeFloat | None = Field(
        None,
        description="Set the fade rate in steps per second.",
        alias="fadeRate",
    )
    save_to_scene: NonNegativeInt | None = Field(
        None, alias="saveToScene", ge=0, lt=16
    )
    color_rgb: ColorRGBData | None = Field(None, alias="colorRGB")
    dimmable_rgb: DimmableRGBData | None = Field(None, alias="dimmableRGB")
    color_rgb_with_fade: ColorWithFadeTime[ColorRGBData] | None = Field(
        None, alias="colorRGBWithFade"
    )
    color_waf: ColorWAFData | None = Field(None, alias="colorWAF")
    dimmable_waf: DimmableWAFData | None = Field(None, alias="dimmableWAF")
    color_waf_with_fade: ColorWithFadeTime[ColorWAFData] | None = Field(
        None, alias="colorWAFWithFade"
    )
    color_kelvin: NonNegativeInt | None = Field(
        None, alias="colorKelvin", gt=15, le=1000000
    )
    dimmable_kelvin: DimmableKelvinData | None = Field(None, alias="dimmableKelvin")
    color_kelvin_with_fade: ColorWithFadeTime[NonNegativeFloat] | None = Field(
        None, alias="colorKelvinWithFade"
    )
    color_xy: ColorXYData | None = Field(None, alias="colorXY")
    dimmable_xy: DimmableXYData | None = Field(None, alias="dimmableXY")
    color_xy_with_fade: ColorWithFadeTime[ColorXYData] | None = Field(
        None, alias="colorXYWithFade"
    )

    model_config = ConfigDict(
        json_schema_extra={
            "examples": [
                {
                    "gotoLastActive": True,
                    "gotoLastActiveWithFade": FadeTime(fadeTime=1.0),
                    "scene": 15,
                    "sceneWithFade": SceneWithFadeData(scene=15, fadeTime=1.0),
                    "fadeTime": 1.0,
                    "fadeRate": 15.8,
                }
            ]
        }
    )



from typing import ClassVar, Final

from lunatone_rest_api_client import Auth
from lunatone_rest_api_client.base import Controllable
from lunatone_rest_api_client.models import (
    ControlData,
    DeviceData,
    DevicesData,
    DeviceUpdateData,
)

_PATH: Final[str] = "devices"


class Device(Controllable):
    """Class that represents a Device object in the API."""

    base_path: ClassVar[str] = _PATH[:-1]

    def __init__(self, auth: Auth, data: DeviceData) -> None:
        """Initialize a device object."""
        self._auth = auth
        self._data = data

    @property
    def path(self) -> str:
        """Return the resource path."""
        return f"{self.base_path}/{self.id}"

    @property
    def data(self) -> DeviceData:
        """Return the raw device data."""
        return self._data

    @property
    def id(self) -> int:
        """Return the ID of the device."""
        return self.data.id

    @property
    def name(self) -> str:
        """Return the name of the device."""
        return self.data.name

    @property
    def is_on(self) -> bool:
        """Return if the light is on."""
        return (
            self.data.features.switchable.status
            if self.data.features.switchable
            else False
        )

    @property
    def brightness(self) -> float:
        """Return the brightness of the light."""
        return (
            self.data.features.dimmable.status if self.data.features.dimmable else 0.0
        )

    @property
    def color_temperature(self) -> int:
        """Return the color temperature of the light in kelvin."""
        return (
            self.data.features.color_kelvin.status
            if self.data.features.color_kelvin
            else 0
        )

    @property
    def rgb_color(self) -> tuple[float, float, float]:
        """Return the RGB color of the light as tuple."""
        rgb_color = (0.0, 0.0, 0.0)
        if self.data.features.color_rgb:
            rgb_color = (
                self.data.features.color_rgb.status.red or 0.0,
                self.data.features.color_rgb.status.green or 0.0,
                self.data.features.color_rgb.status.blue or 0.0,
            )
        return rgb_color

    @property
    def rgbw_color(self) -> tuple[float, float, float, float]:
        """Return the RGBW color of the light as tuple."""
        rgbw_color = (0.0, 0.0, 0.0, 0.0)
        if self.data.features.color_waf:
            rgbw_color = (
                *self.rgb_color,
                self.data.features.color_waf.status.white or 0.0,
            )
        return rgbw_color

    @property
    def xy_color(self) -> tuple[float, float]:
        """Return the XY color of the light as tuple."""
        xy_color = (0.0, 0.0)
        if self.data.features.color_xy:
            xy_color = (
                self.data.features.color_xy.status.x or 0.0,
                self.data.features.color_xy.status.y or 0.0,
            )
        return xy_color

    @property
    def is_dimmable(self) -> bool:
        """Return if the light is dimmable."""
        return True  # At the moment this is not possible to determine in the REST API

    async def async_update(self, data: DeviceUpdateData | None = None) -> None:
        """Update the device data."""
        if data is not None:
            json_data = data.model_dump(by_alias=True, exclude_none=True)
            response = await self._auth.put(self.path, json=json_data)
        else:
            response = await self._auth.get(self.path)
        self._data = DeviceData.model_validate(await response.json())

    async def async_control(self, data: ControlData) -> None:
        """Control the device."""
        json_data = data.model_dump(by_alias=True, exclude_none=True)
        await self._auth.post(f"{self.path}/control", json=json_data)


class Devices:
    """Class that represents a Devices object in the API."""

    path: ClassVar[str] = _PATH

    def __init__(self, auth: Auth) -> None:
        """Initialize a Devices object."""
        self._auth = auth
        self._data: DevicesData | None = None

    @property
    def data(self) -> DevicesData | None:
        """Return the raw devices data."""
        return self._data

    @property
    def devices(self) -> list[Device]:
        """Return a list of devices."""
        return (
            [Device(self._auth, device) for device in self.data.devices]
            if self.data
            else []
        )

    async def async_update(self) -> None:
        """Update the devices data."""
        response = await self._auth.get(self.path)
        self._data = DevicesData.model_validate(await response.json())



from enum import StrEnum

from pydantic import BaseModel, Field, NonNegativeInt


class SensorMeasurementUnit(StrEnum):
    NONE = ""
    DEGREE_CELSIUS = "°C"
    PERCENT = "%"
    HECTOPASCAL = "hPa"
    PARTS_PER_MILLION = "ppm"
    PARTS_PER_BILLION = "ppb"
    INDOOR_AIR_QUALITY = "AQI"
    LUX = "lx"


class SensorType(StrEnum):
    UNKNOWN = "unknown"
    TEMPERATURE = "temperature"
    AIR_HUMIDITY = "airHumidity"
    AIR_PRESSURE = "airPressure"
    ECO2 = "eCO2"
    VOC = "VOC"
    AIR_QUALITY = "airQuality"
    LIGHT = "light"
    OCCUPANCY = "occupancy"


class SensorAddressType(StrEnum):
    INTERNAL = "internal"
    DALI = "dali"


class SensorDaliAddress(BaseModel):
    line: int
    address: int
    instanceNumber: int


class SensorData(BaseModel):
    id: NonNegativeInt
    name: str = ""
    unit: SensorMeasurementUnit = SensorMeasurementUnit.NONE
    type: SensorType = SensorType.UNKNOWN
    value: float | None = None
    timestamp: str | None = None
    address_type: SensorAddressType | None = Field(None, alias="addressType")
    dali_sensor_address: SensorDaliAddress | None = Field(None, alias="daliSensorAddress")


class SensorsData(BaseModel):
    sensors: list[SensorData]



import aiohttp
import pytest
from aioresponses.core import aioresponses

from lunatone_rest_api_client import Auth, Sensor, Sensors
from lunatone_rest_api_client.models import SensorData, SensorsData

from .common import SENSORS_DATA_RAW


@pytest.fixture
def sensors_data() -> SensorsData:
    return SensorsData.model_validate(SENSORS_DATA_RAW)


@pytest.fixture
def sensor_data(sensors_data: SensorsData) -> SensorData:
    for sensor in sensors_data.sensors:
        return sensor


@pytest.mark.asyncio
async def test_sensors_update(
    aioresponses: aioresponses, base_url: str, sensors_data: SensorsData
) -> None:
    json_data = sensors_data.model_dump(by_alias=True)
    aioresponses.get(f"{base_url}/{Sensors.path}", payload=json_data)

    async with aiohttp.ClientSession() as session:
        sensors = Sensors(Auth(session, base_url))
        await sensors.async_update()
        assert sensors.data == sensors_data


@pytest.mark.asyncio
async def test_sensor_update(
    aioresponses: aioresponses, base_url: str, sensor_data: SensorData
) -> None:
    json_data = sensor_data.model_dump(by_alias=True)
    aioresponses.get(
        f"{base_url}/{Sensor.base_path}/{sensor_data.id}", payload=json_data
    )

    async with aiohttp.ClientSession() as session:
        sensors = Sensor(Auth(session, base_url), SensorData(id=sensor_data.id))
        await sensors.async_update()
        assert sensors.data == sensor_data



from typing import ClassVar

from lunatone_rest_api_client import Auth
from lunatone_rest_api_client.base import Controllable
from lunatone_rest_api_client.models import (
    ControlData,
    ScanData,
    ScanState,
    StartScanData,
)


class DALIScan:
    """Class that represents a DALI Scan object in the API."""

    path: ClassVar[str] = "dali/scan"

    def __init__(self, auth: Auth) -> None:
        self._auth = auth
        self._data = ScanData()

    @property
    def data(self) -> ScanData:
        """Return the raw scan data."""
        return self._data

    @property
    def status(self) -> ScanState:
        """Return the scan status."""
        return self.data.status

    @property
    def is_busy(self) -> bool:
        """Return if the scan is in progress."""
        return (
            self.status == ScanState.ADDRESSING or self.status == ScanState.IN_PROGRESS
        )

    @property
    def progress(self) -> float:
        """Return a progress value in percent."""
        return self.data.progress if self.data.progress is not None else 0.0

    async def async_update(self) -> None:
        """Update the DALI scan."""
        response = await self._auth.get(self.path)
        self._data = ScanData.model_validate(await response.json())

    async def async_start(self, data: StartScanData) -> None:
        """Start the DALI scan."""
        json_data = data.model_dump(by_alias=True, exclude_none=True)
        response = await self._auth.post(self.path, json=json_data)
        self._data = ScanData.model_validate(await response.json())

    async def async_cancel(self) -> None:
        """Cancel the DALI scan."""
        response = await self._auth.post(f"{self.path}/cancel")
        self._data = ScanData.model_validate(await response.json())


class DALIBroadcast(Controllable):
    """Class that represents a DALI broadcast object in the API."""

    path: ClassVar[str] = "broadcast"

    def __init__(self, auth: Auth, line: int | None = None) -> None:
        self._auth = auth
        self._line = line

    @property
    def line(self) -> int | None:
        """Return the DALI line number."""
        return self._line

    async def async_control(self, data: ControlData) -> None:
        """Control the DALI broadcast."""
        json_data = data.model_dump(by_alias=True, exclude_none=True)
        await self._auth.post(
            f"{self.path}/control",
            json=json_data,
            params={"_line": self.line} if self.line is not None else None,
        )


class DALIGroup(Controllable):
    """Class that represents a DALI group object in the API."""

    path: ClassVar[str] = "group"

    def __init__(self, auth: Auth, group_number: int, line: int | None = None) -> None:
        self._auth = auth
        self._line = line
        self._number = group_number

    @property
    def line(self) -> int | None:
        """Return the DALI line number."""
        return self._line

    @property
    def number(self) -> int:
        """Return the DALI group number."""
        return self._number

    async def async_control(self, data: ControlData) -> None:
        """Control the DALI group."""
        json_data = data.model_dump(by_alias=True, exclude_none=True)
        await self._auth.post(
            f"{self.path}/{self.number}/control",
            json=json_data,
            params={"_line": self.line} if self.line is not None else None,
        )



import aiohttp
import pytest
from aioresponses.core import aioresponses

from lunatone_rest_api_client import Auth, DALIBroadcast, DALIGroup, DALIScan
from lunatone_rest_api_client.models import (
    ControlData,
    ScanData,
    ScanState,
    StartScanData,
)


@pytest.fixture
def dali_scan_data_init() -> ScanData:
    return ScanData()


@pytest.mark.asyncio
async def test_scan_update(
    aioresponses: aioresponses, base_url: str, dali_scan_data_init: ScanData
) -> None:
    expected_scan_data = ScanData(
        id="2bcdf829-2770-4a98-b1c7-b1ca4bac408f",
        progress=100,
        found=0,
        foundSensors=0,
        status=ScanState.DONE,
    )

    aioresponses.get(
        f"{base_url}/{DALIScan.path}",
        payload=dali_scan_data_init.model_dump(by_alias=True),
    )
    aioresponses.get(
        f"{base_url}/{DALIScan.path}",
        payload=expected_scan_data.model_dump(by_alias=True),
    )

    async with aiohttp.ClientSession() as session:
        scan = DALIScan(Auth(session, base_url))

        await scan.async_update()
        assert scan.data == dali_scan_data_init

        await scan.async_update()
        assert scan.data == expected_scan_data


@pytest.mark.asyncio
async def test_scan_start(
    aioresponses: aioresponses, base_url: str, dali_scan_data_init: ScanData
) -> None:
    expected_scan_data = ScanData(
        id="2bcdf829-2770-4a98-b1c7-b1ca4bac408f",
        progress=0,
        found=0,
        foundSensors=0,
        status=ScanState.NOT_STARTED,
    )

    aioresponses.get(
        f"{base_url}/{DALIScan.path}",
        payload=dali_scan_data_init.model_dump(by_alias=True),
    )
    aioresponses.post(
        f"{base_url}/{DALIScan.path}",
        payload=expected_scan_data.model_dump(by_alias=True),
    )

    async with aiohttp.ClientSession() as session:
        scan = DALIScan(Auth(session, base_url))

        await scan.async_update()
        assert scan.data == dali_scan_data_init

        await scan.async_start(
            StartScanData(newInstallation=False, noAddressing=False, useLines=[])
        )
        assert scan.data == expected_scan_data


@pytest.mark.asyncio
async def test_scan_cancel(
    aioresponses: aioresponses, base_url: str, dali_scan_data_init: ScanData
) -> None:
    expected_scan_data = ScanData(
        id="2bcdf829-2770-4a98-b1c7-b1ca4bac408f",
        progress=0,
        found=0,
        foundSensors=0,
        status=ScanState.CANCELLED,
    )

    aioresponses.get(
        f"{base_url}/{DALIScan.path}",
        payload=dali_scan_data_init.model_dump(by_alias=True),
    )
    aioresponses.post(
        f"{base_url}/{DALIScan.path}/cancel",
        payload=expected_scan_data.model_dump(by_alias=True),
    )

    async with aiohttp.ClientSession() as session:
        scan = DALIScan(Auth(session, base_url))

        await scan.async_update()
        assert scan.data == dali_scan_data_init

        await scan.async_cancel()
        assert scan.data == expected_scan_data


@pytest.mark.asyncio
async def test_control_broadcast(aioresponses: aioresponses, base_url: str) -> None:
    aioresponses.post(
        f"{base_url}/broadcast/control",
        status=204,
    )

    async with aiohttp.ClientSession() as session:
        dali_control = DALIBroadcast(Auth(session, base_url))

        try:
            await dali_control.async_control(ControlData(switchable=True))
        except aiohttp.ClientError:
            assert False


@pytest.mark.asyncio
async def test_control_broadcast_should_throw_error(
    aioresponses: aioresponses, base_url: str
) -> None:
    aioresponses.post(
        f"{base_url}/{DALIBroadcast.path}/control",
        status=500,
    )

    async with aiohttp.ClientSession() as session:
        dali_control = DALIBroadcast(Auth(session, base_url))

        with pytest.raises(aiohttp.ClientResponseError):
            await dali_control.async_control(ControlData(switchable=True))


@pytest.mark.asyncio
@pytest.mark.parametrize("group_number", list(range(16)))
async def test_control_group(
    aioresponses: aioresponses, base_url: str, group_number: int
) -> None:
    aioresponses.post(
        f"{base_url}/{DALIGroup.path}/{group_number}/control",
        status=204,
    )

    async with aiohttp.ClientSession() as session:
        dali_control = DALIGroup(Auth(session, base_url), group_number)

        try:
            await dali_control.async_control(ControlData(switchable=True))
        except aiohttp.ClientError:
            assert False


@pytest.mark.asyncio
@pytest.mark.parametrize("group_number", list(range(16)))
async def test_control_group_should_throw_error(
    aioresponses: aioresponses, base_url: str, group_number: int
) -> None:
    aioresponses.post(
        f"{base_url}/{DALIGroup.path}/{group_number}/control",
        status=500,
    )

    async with aiohttp.ClientSession() as session:
        dali_control = DALIGroup(Auth(session, base_url), group_number)

        with pytest.raises(aiohttp.ClientResponseError):
            await dali_control.async_control(ControlData(switchable=True))



from pydantic import BaseModel, Field, PositiveInt, NonNegativeInt

from lunatone_rest_api_client.models import FeaturesStatus, TimeSignature


class ZoneTarget(BaseModel):
    type: str
    id: NonNegativeInt | None = None


class ZoneData(BaseModel):
    id: PositiveInt
    name: str = ""
    targets: list[ZoneTarget] = []
    features: FeaturesStatus = FeaturesStatus()
    time_signature: TimeSignature | None = Field(None, alias="timeSignature")


class ZonesData(BaseModel):
    zones: list[ZoneData]
    time_signature: TimeSignature | None = Field(None, alias="timeSignature")



from lunatone_rest_api_client.auth import Auth
from lunatone_rest_api_client.dali import DALIBroadcast, DALIGroup, DALIScan
from lunatone_rest_api_client.devices import Device, Devices
from lunatone_rest_api_client.info import Info
from lunatone_rest_api_client.sensors import Sensor, Sensors
from lunatone_rest_api_client.zones import Zone, Zones

__all__ = [
    "Auth",
    "DALIBroadcast",
    "DALIGroup",
    "DALIScan",
    "Device",
    "Devices",
    "Info",
    "Sensor",
    "Sensors",
    "Zone",
    "Zones",
]



from lunatone_rest_api_client.models.control import ControlData
from lunatone_rest_api_client.models.common import (
    DALIType,
    FeaturesStatus,
    TimeSignature,
)
from lunatone_rest_api_client.models.dali import ScanData, ScanState, StartScanData
from lunatone_rest_api_client.models.devices import (
    DevicesData,
    DeviceData,
    DeviceUpdateData,
)
from lunatone_rest_api_client.models.info import (
    DALIBusData,
    DescriptorData,
    DeviceInfoData,
    InfoData,
    LineStatus,
)
from lunatone_rest_api_client.models.sensors import (
    SensorAddressType,
    SensorDaliAddress,
    SensorData,
    SensorsData,
    SensorMeasurementUnit,
    SensorType,
)
from lunatone_rest_api_client.models.zones import (
    ZoneData,
    ZonesData,
    ZoneTarget,
)

__all__ = [
    "ControlData",
    "DALIType",
    "FeaturesStatus",
    "TimeSignature",
    "ScanData",
    "ScanState",
    "StartScanData",
    "DevicesData",
    "DeviceData",
    "DeviceUpdateData",
    "DALIBusData",
    "DescriptorData",
    "DeviceInfoData",
    "InfoData",
    "LineStatus",
    "SensorAddressType",
    "SensorDaliAddress",
    "SensorData",
    "SensorsData",
    "SensorMeasurementUnit",
    "SensorType",
    "ZoneData",
    "ZonesData",
    "ZoneTarget",
]



from pydantic import BaseModel, Field, NonNegativeInt, PositiveInt

from lunatone_rest_api_client.models import (
    DALIType,
    FeaturesStatus,
    TimeSignature,
)


class DeviceStatus(BaseModel):
    raw: NonNegativeInt = 0
    control_gear_failure: bool = Field(False, alias="controlGearFailure")
    lamp_failure: bool = Field(False, alias="lampFailure")
    lamp_on: bool = Field(False, alias="lampOn")
    limit_error: bool = Field(False, alias="limitError")
    fade_running: bool = Field(False, alias="fadeRunning")
    reset_state: bool = Field(False, alias="resetState")
    is_unaddressed: bool = Field(False, alias="isUnaddressed")
    power_cycle_see: bool = Field(False, alias="powerCycleSeen")


class DeviceData(BaseModel):
    id: PositiveInt
    name: str = ""
    type: str = "default"
    available: bool = False
    status: DeviceStatus = DeviceStatus()
    features: FeaturesStatus = FeaturesStatus()
    scenes: list = []
    groups: list[NonNegativeInt] = []
    address: NonNegativeInt
    line: NonNegativeInt
    dali_types: list[DALIType | NonNegativeInt] = Field([], alias="daliTypes")
    time_signature: TimeSignature | None = Field(None, alias="timeSignature")


class DevicesData(BaseModel):
    devices: list[DeviceData]
    time_signature: TimeSignature | None = Field(None, alias="timeSignature")


class DeviceUpdateData(BaseModel):
    name: str
    groups: list[NonNegativeInt] | None = None



import argparse
import asyncio

from lunatone_rest_api_client.discovery import async_discover_devices_stream


parser = argparse.ArgumentParser(prog="Discovery")
parser.add_argument(
    "-i", "--ip", type=str, help="Specify the IP address of a network interface"
)
arguments = parser.parse_args()


async def main():
    loop = asyncio.get_running_loop()
    async for dev in async_discover_devices_stream(
        loop,
        timeout=4.0,
        local_ip=arguments.ip,
    ):
        print(dev)


if __name__ == "__main__":
    asyncio.run(main())



from abc import ABC, abstractmethod

from lunatone_rest_api_client.models import ControlData
from lunatone_rest_api_client.models.common import ColorRGBData, ColorWAFData


class Controllable(ABC):
    """Class that provides basic control methods."""

    async def switch_on(self) -> None:
        """Switch on the device."""
        await self.async_control(ControlData(switchable=True))

    async def switch_off(self) -> None:
        """Switch off the device."""
        await self.async_control(ControlData(switchable=False))

    async def fade_to_brightness(self, brightness: float) -> None:
        """Fade to provided brightness level."""
        await self.async_control(ControlData(dimmable=brightness))

    async def fade_to_last_active_level(self) -> None:
        """Fade to last active level."""
        await self.async_control(ControlData(gotoLastActive=True))

    async def fade_to_color_temperature(self, color_temperature: int) -> None:
        """Fade to color temperature."""
        await self.async_control(ControlData(colorKelvin=color_temperature))

    async def fade_to_rgbw_color(
        self, rgb_color: tuple[float, float, float], white: float | None = None
    ) -> None:
        """Fade to RGBW color."""
        if white is not None:
            await self.async_control(ControlData(colorWAF=ColorWAFData(w=white)))
        await self.async_control(
            ControlData(
                colorRGB=ColorRGBData(r=rgb_color[0], g=rgb_color[1], b=rgb_color[2])
            )
        )

    @abstractmethod
    async def async_control(self, data: ControlData) -> None:
        """Control the instance."""
